#include "stdafx.h"
#include <windows.h>

#using <mscorlib.dll>
#using "TCONNECTOR2Lib.dll"				
#include <tchar.h>

using namespace System;
using namespace System::IO;
using namespace System::Runtime::InteropServices;
using namespace System::Threading;
using namespace TCONNECTOR2Lib;

// ---------------------------------------------------------
// First import/convert the type library to a .NET assembly,
// --> look at our short tutorial "HowTo.html" (included).

// ---------------------------------------------------------
// event handler class (with your code for the OnData event)
__gc class TCEventHandler
{
public:
	TCEventHandler()	{	};
	~TCEventHandler()	{   };

	void OnData(String* sData)
	{
		Console::WriteLine(S"Data: {0}", sData);
	};
};

// --------------------------------------------------------
// main

int _tmain(void)
{
	// if you want to use OnData event, use Multi Threaded Apartment (set by default)
	// Thread::CurrentThread->ApartmentState  = System::Threading::ApartmentState::MTA;
	// --
	// if you don't use the OnData event, use Single Threaded Apartment with this command
	// Thread::CurrentThread->ApartmentState  = System::Threading::ApartmentState::STA;
	
	Console::WriteLine(S"Application started.");

	try 
	{ 
		// create new TConnector2 object
		TConnector2Class *pTC = new TConnector2Class();	

		// CONNECTION SETTINGS 
		// ----- FILE
		// pTC->IOType = TCONNECTOR2Lib::e_DeviceType::eFILE; // set type of I/O to File
		// pTC->Device = S"c:\\testdata.txt"; 				// specify filename
		// pTC->NoOfBytes = 5;
		// ----- TCP:
		// pTC->IOType = TCONNECTOR2Lib::e_DeviceType::eTCP; 		// set type of I/O to TCP
		// pTC->Host = S"193.171.34.240"; 		// specify hostname and port (port 13 delivers the daytime)
		// pTC->Service = S"13";
		
		// ----- COM2 (serial port)
		pTC->IOType = TCONNECTOR2Lib::e_DeviceType::eCOM;
		pTC->Device = S"COM2";
		pTC->Baud   = S"9600";
		pTC->Parity = S"N";
		pTC->Stop   = S"1";
		pTC->RTSCTS = true;

		Console::WriteLine(S"Opening {0}...", pTC->Device);

		// open connection
		pTC->Open(); // if port not accessable we get an exception

		// Now two possibilities to read in data:

		///////////////////////////////////////////////
		// (1.) Synchronous mode (for read and write)
		///////////////////////////////////////////////
		try
		{
			Console::WriteLine(S"Sync mode: waiting for data (5 seconds) from {0}", pTC->Device);
			
			// read max 100 bytes of data within timeout interval of 5000 ms
			String *sData = pTC->Read(100, 5000);
			int    nCount = pTC->GetNoOfBytesRead();

			Console::WriteLine(S"Received data: {0}", sData);
			Console::WriteLine(S"Bytes read:    {0}", nCount.ToString("G"));

			// write data to port or file
			// String* sTxData	= S"MyData1234";
			// int     nTimeout	= 5000;
			// int nBytesWritten = pTC->Write(sTxData->Length, nTimeout, sTxData);
		}
		catch (COMException * pE) 
		{ 
			// handle timeout exception if no data was received
			if (HRESULT_CODE(pE->ErrorCode) == ERROR_TIMEOUT)
				Console::WriteLine(S"No data was received during timeout period (5 seconds)");
			else
                Console::WriteLine(S"Exception: {0}", pE->Message);
		}
		catch (Exception * pE)
		{
			Console::WriteLine(S"Exception: {0}", pE->Message);
		}

		///////////////////////////////////////
		// (2.) Asynchronous mode (event based)
		///////////////////////////////////////

		// your event handler class
		TCEventHandler* evh = new TCEventHandler();
		 // create TConnector Event Handler for OnData event
		TCONNECTOR2Lib::_ITConnectorEvents_OnDataEventHandler* pH = new TCONNECTOR2Lib::_ITConnectorEvents_OnDataEventHandler(evh, &TCEventHandler::OnData);
		// register event handler for actual TConnector instance
		pTC->add_OnData(pH);

		// async mode settings for "data packaging" for one event:
		pTC->NoOfBytes = 30;			// portions of max 30 Bytes per event
		pTC->Timeout   = 300;   		// collect data during 300 ms timeout period, then fire event
		pTC->Timeout_Infinite = false;	// false: enable timeout; true: disable timeout, collect data until NoOfBytes reached

		// activate OnData event and "listen" for data
		pTC->StartListen();		
		
		// wait for abort (key)
		Console::WriteLine();
		Console::WriteLine(S"Async mode: OnData event activated (press a key to abort!)");
		Console::WriteLine(S"Now waiting for data from {0}", pTC->Device);
		Console::Read();

		pTC->StopListen();		// stop OnData event
		pTC->remove_OnData(pH); // unregister event handler
		delete evh;				// free memory

		pTC->Close(); 			// close connection
	} 
	catch (FileNotFoundException * pE) 
	{ 
		//during open: if port or file not exists we get an exception
		Console::WriteLine(S"File or port not found!");
	}
	catch (UnauthorizedAccessException * pE) 
	{ 
		// during open: port may be used by another application
		Console::WriteLine(S"Access denied (make sure that no other application uses this file or port!)");
	}
	catch (Exception * pE) 
	{
		Console::WriteLine(S"Exception: {0}", pE->Message);
	}

	Console::WriteLine(S"Application terminated - press a key...");
	Console::Read();    // wait for a key

    return 0;
}

